# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class Job(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'ray_session_name': 'str',
        'ray_job_id': 'str',
        'name': 'str',
        'status': 'JobStatus',
        'created_at': 'datetime',
        'finished_at': 'datetime',
        'ray_job_submission_id': 'str',
        'cluster_id': 'str',
        'namespace_id': 'str',
        'runtime_environment_id': 'str',
        'project_id': 'str',
        'creator_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'ray_session_name': 'ray_session_name',
        'ray_job_id': 'ray_job_id',
        'name': 'name',
        'status': 'status',
        'created_at': 'created_at',
        'finished_at': 'finished_at',
        'ray_job_submission_id': 'ray_job_submission_id',
        'cluster_id': 'cluster_id',
        'namespace_id': 'namespace_id',
        'runtime_environment_id': 'runtime_environment_id',
        'project_id': 'project_id',
        'creator_id': 'creator_id'
    }

    def __init__(self, id=None, ray_session_name=None, ray_job_id=None, name=None, status=None, created_at=None, finished_at=None, ray_job_submission_id=None, cluster_id=None, namespace_id=None, runtime_environment_id=None, project_id=None, creator_id=None, local_vars_configuration=None):  # noqa: E501
        """Job - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._ray_session_name = None
        self._ray_job_id = None
        self._name = None
        self._status = None
        self._created_at = None
        self._finished_at = None
        self._ray_job_submission_id = None
        self._cluster_id = None
        self._namespace_id = None
        self._runtime_environment_id = None
        self._project_id = None
        self._creator_id = None
        self.discriminator = None

        self.id = id
        self.ray_session_name = ray_session_name
        self.ray_job_id = ray_job_id
        if name is not None:
            self.name = name
        self.status = status
        self.created_at = created_at
        if finished_at is not None:
            self.finished_at = finished_at
        if ray_job_submission_id is not None:
            self.ray_job_submission_id = ray_job_submission_id
        self.cluster_id = cluster_id
        self.namespace_id = namespace_id
        self.runtime_environment_id = runtime_environment_id
        if project_id is not None:
            self.project_id = project_id
        self.creator_id = creator_id

    @property
    def id(self):
        """Gets the id of this Job.  # noqa: E501

        Server assigned unique identifier.  # noqa: E501

        :return: The id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Job.

        Server assigned unique identifier.  # noqa: E501

        :param id: The id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def ray_session_name(self):
        """Gets the ray_session_name of this Job.  # noqa: E501

        Name of the Session provided from Ray  # noqa: E501

        :return: The ray_session_name of this Job.  # noqa: E501
        :rtype: str
        """
        return self._ray_session_name

    @ray_session_name.setter
    def ray_session_name(self, ray_session_name):
        """Sets the ray_session_name of this Job.

        Name of the Session provided from Ray  # noqa: E501

        :param ray_session_name: The ray_session_name of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ray_session_name is None:  # noqa: E501
            raise ValueError("Invalid value for `ray_session_name`, must not be `None`")  # noqa: E501

        self._ray_session_name = ray_session_name

    @property
    def ray_job_id(self):
        """Gets the ray_job_id of this Job.  # noqa: E501

        ID of the Job provided from Ray  # noqa: E501

        :return: The ray_job_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._ray_job_id

    @ray_job_id.setter
    def ray_job_id(self, ray_job_id):
        """Sets the ray_job_id of this Job.

        ID of the Job provided from Ray  # noqa: E501

        :param ray_job_id: The ray_job_id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ray_job_id is None:  # noqa: E501
            raise ValueError("Invalid value for `ray_job_id`, must not be `None`")  # noqa: E501

        self._ray_job_id = ray_job_id

    @property
    def name(self):
        """Gets the name of this Job.  # noqa: E501

        Name of this Job.  # noqa: E501

        :return: The name of this Job.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Job.

        Name of this Job.  # noqa: E501

        :param name: The name of this Job.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def status(self):
        """Gets the status of this Job.  # noqa: E501

        Status of this Job's execution.  # noqa: E501

        :return: The status of this Job.  # noqa: E501
        :rtype: JobStatus
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Job.

        Status of this Job's execution.  # noqa: E501

        :param status: The status of this Job.  # noqa: E501
        :type: JobStatus
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def created_at(self):
        """Gets the created_at of this Job.  # noqa: E501

        Time at which this Job was created.  # noqa: E501

        :return: The created_at of this Job.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Job.

        Time at which this Job was created.  # noqa: E501

        :param created_at: The created_at of this Job.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def finished_at(self):
        """Gets the finished_at of this Job.  # noqa: E501

        Time at which this Job finished. If absent, this Job is still running.  # noqa: E501

        :return: The finished_at of this Job.  # noqa: E501
        :rtype: datetime
        """
        return self._finished_at

    @finished_at.setter
    def finished_at(self, finished_at):
        """Sets the finished_at of this Job.

        Time at which this Job finished. If absent, this Job is still running.  # noqa: E501

        :param finished_at: The finished_at of this Job.  # noqa: E501
        :type: datetime
        """

        self._finished_at = finished_at

    @property
    def ray_job_submission_id(self):
        """Gets the ray_job_submission_id of this Job.  # noqa: E501

        ID of the submitted Ray Job that this Job corresponds to.  # noqa: E501

        :return: The ray_job_submission_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._ray_job_submission_id

    @ray_job_submission_id.setter
    def ray_job_submission_id(self, ray_job_submission_id):
        """Sets the ray_job_submission_id of this Job.

        ID of the submitted Ray Job that this Job corresponds to.  # noqa: E501

        :param ray_job_submission_id: The ray_job_submission_id of this Job.  # noqa: E501
        :type: str
        """

        self._ray_job_submission_id = ray_job_submission_id

    @property
    def cluster_id(self):
        """Gets the cluster_id of this Job.  # noqa: E501

        ID of the Anyscale Cluster this Job is on.  # noqa: E501

        :return: The cluster_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this Job.

        ID of the Anyscale Cluster this Job is on.  # noqa: E501

        :param cluster_id: The cluster_id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cluster_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cluster_id`, must not be `None`")  # noqa: E501

        self._cluster_id = cluster_id

    @property
    def namespace_id(self):
        """Gets the namespace_id of this Job.  # noqa: E501

        ID of the Anyscale Namespace this Job is using.  # noqa: E501

        :return: The namespace_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._namespace_id

    @namespace_id.setter
    def namespace_id(self, namespace_id):
        """Sets the namespace_id of this Job.

        ID of the Anyscale Namespace this Job is using.  # noqa: E501

        :param namespace_id: The namespace_id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and namespace_id is None:  # noqa: E501
            raise ValueError("Invalid value for `namespace_id`, must not be `None`")  # noqa: E501

        self._namespace_id = namespace_id

    @property
    def runtime_environment_id(self):
        """Gets the runtime_environment_id of this Job.  # noqa: E501

        ID of the Anyscale Runtime Environment this Job is using.  # noqa: E501

        :return: The runtime_environment_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._runtime_environment_id

    @runtime_environment_id.setter
    def runtime_environment_id(self, runtime_environment_id):
        """Sets the runtime_environment_id of this Job.

        ID of the Anyscale Runtime Environment this Job is using.  # noqa: E501

        :param runtime_environment_id: The runtime_environment_id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and runtime_environment_id is None:  # noqa: E501
            raise ValueError("Invalid value for `runtime_environment_id`, must not be `None`")  # noqa: E501

        self._runtime_environment_id = runtime_environment_id

    @property
    def project_id(self):
        """Gets the project_id of this Job.  # noqa: E501

        ID of the Project this Job belongs to.  # noqa: E501

        :return: The project_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this Job.

        ID of the Project this Job belongs to.  # noqa: E501

        :param project_id: The project_id of this Job.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def creator_id(self):
        """Gets the creator_id of this Job.  # noqa: E501

        ID of the user who created this Job.  # noqa: E501

        :return: The creator_id of this Job.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this Job.

        ID of the user who created this Job.  # noqa: E501

        :param creator_id: The creator_id of this Job.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and creator_id is None:  # noqa: E501
            raise ValueError("Invalid value for `creator_id`, must not be `None`")  # noqa: E501

        self._creator_id = creator_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Job):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Job):
            return True

        return self.to_dict() != other.to_dict()



from datetime import datetime
from typing import Optional

from anyscale.api import get_anyscale_api_client, get_api_client
from anyscale.cli_logger import _CliLogger
from anyscale.client.openapi_client.api.default_api import DefaultApi
from anyscale.cloud import get_cloud_id_and_name, get_last_used_cloud
from anyscale.sdk.anyscale_client import (
    ComputeTemplate,
    ComputeTemplateConfig,
    ComputeTemplateQuery,
    CreateComputeTemplate,
)
from anyscale.utils.cloud_utils import get_organization_default_cloud


log = _CliLogger()


def get_default_cluster_compute(
    cloud_name: Optional[str],
    project_id: Optional[str],
    api_client: Optional[DefaultApi] = None,
    anyscale_api_client: Optional[DefaultApi] = None,
) -> ComputeTemplate:
    if api_client is None:
        api_client = get_api_client()
    if anyscale_api_client is None:
        anyscale_api_client = get_anyscale_api_client()

    if cloud_name is None:
        default_cloud_name = get_organization_default_cloud(api_client)
        if default_cloud_name:
            cloud_name = default_cloud_name
        else:
            cloud_name = get_last_used_cloud(project_id, anyscale_api_client)

    cloud_id, _ = get_cloud_id_and_name(api_client, cloud_name=cloud_name)
    config_object = anyscale_api_client.get_default_compute_config(cloud_id).result
    compute_template = register_compute_template(config_object)
    return compute_template


def get_cluster_compute_from_name(
    cluster_compute_name: str, api_client: Optional[DefaultApi] = None,
) -> ComputeTemplate:
    if api_client is None:
        api_client = get_api_client()
    cluster_computes = api_client.search_compute_templates_api_v2_compute_templates_search_post(
        ComputeTemplateQuery(
            orgwide=True, name={"equals": cluster_compute_name}, include_anonymous=True,
        )
    ).results

    if len(cluster_computes) == 0:
        raise ValueError(
            f"The cluster compute template {cluster_compute_name}" " is not registered."
        )
    return cluster_computes[0]


def register_compute_template(
    config_object: ComputeTemplateConfig, api_client: Optional[DefaultApi] = None,
) -> ComputeTemplate:
    """
    Register compute template with a default name and return the compute template id."""
    if api_client is None:
        api_client = get_api_client()
    created_template = api_client.create_compute_template_api_v2_compute_templates_post(
        create_compute_template=CreateComputeTemplate(
            name="cli-autogenerated-config-{}".format(datetime.now().isoformat()),
            config=config_object,
            anonymous=True,
        )
    ).result
    return created_template



import sys
from typing import Any, Tuple


def detect_ray_version(imported_ray: Any = None) -> Tuple[str, str]:
    """
    Returns Tuple of ray_version, ray_commit
    Raises RuntimeError if imported_ray is not provided and ray is not installed
    """
    if imported_ray is None:
        try:
            import ray
        except ModuleNotFoundError:
            raise RuntimeError(
                "Ray is not installed. Please install with: \n"
                "pip install -U --force-reinstall `python -m anyscale.connect required_ray_version`"
            )
        imported_ray = ray

    return imported_ray.__version__, imported_ray.__commit__


def detect_python_minor_version() -> str:
    return f"{sys.version_info[0]}.{sys.version_info[1]}"



import os
from typing import Any, Optional, Tuple

import click

from anyscale.cli_logger import _CliLogger
from anyscale.controllers.project_controller import COMPUTE_CONFIG_FILENAME
from anyscale.controllers.session_controller import SessionController


log = _CliLogger()


@click.command(name="down", help="[DEPRECATED] Stop the current cluster.", hidden=True)
@click.argument("session-name", required=False, default=None)
@click.option(
    "--workers-only", is_flag=True, default=False, help="Only destroy the workers."
)
@click.option(
    "--keep-min-workers",
    is_flag=True,
    default=False,
    help="Retain the minimal amount of workers specified in the config.",
)
@click.option("--delete", help="Delete the cluster as well.", is_flag=True)
@click.option(
    "--skip-snapshot", help="Skip taking snapshot.", is_flag=True, default=False,
)
@click.pass_context
def anyscale_stop(
    ctx: Any,
    session_name: Optional[str],
    workers_only: bool,
    keep_min_workers: bool,
    delete: bool,
    skip_snapshot: bool,
) -> None:
    log.warning(
        "`anyscale down` has been deprecated. Please use `anyscale cluster terminate` instead."
    )
    session_controller = SessionController()
    session_controller.stop(
        session_name,
        workers_only=workers_only,
        keep_min_workers=keep_min_workers,
        delete=delete,
        skip_snapshot=skip_snapshot,
    )


@click.command(
    name="up",
    context_settings=dict(ignore_unknown_options=True,),
    help="DEPRECATED: Start or update a cluster based on the current project configuration.",
    hidden=True,
)
@click.argument("session-name", required=False)
@click.option(
    "--dangerously-set-build-id",
    hidden=True,
    help="Dangerously set the build-id. This is only used by anyscale connect.",
    default=None,
)
@click.option(
    "--config", "config", help="Config to start cluster with.", default=None,
)
@click.option(
    "--build-id",
    help="The build ID generated from the app templates service.",
    # TODO: add the app templates CLI command to the description
    # once the CLI is done.
    default=None,
)
@click.option(
    "--compute-config",
    help=(
        "The JSON file of the compute config to launch this cluster with. "
        "An example can be found at {filename}. "
        "The full JSON schema can be viewed at {website}. ".format(
            filename=COMPUTE_CONFIG_FILENAME, website="api.anyscale.com/v0/docs",
        )
    ),
    default=None,
)
@click.option(
    "--no-restart",
    is_flag=True,
    default=False,
    help=(
        "Whether to skip restarting Ray services during the update. "
        "This avoids interrupting running jobs."
    ),
)
@click.option(
    "--restart-only",
    is_flag=True,
    default=False,
    help=(
        "Whether to skip running setup commands and only restart Ray. "
        "This cannot be used with 'no-restart'."
    ),
)
@click.option(
    "--min-workers",
    required=False,
    type=int,
    help="Override the configured min worker node count for the cluster.",
)
@click.option(
    "--max-workers",
    required=False,
    type=int,
    help="Override the configured max worker node count for the cluster.",
)
@click.option(
    "--disable-sync",
    is_flag=True,
    default=False,
    help=(
        "Disables syncing file mounts and project directory. This is "
        "useful when 'restart-only' is set and file syncing takes a long time."
    ),
)
@click.option("--cloud-id", required=False, help="Id of the cloud to use", default=None)
@click.option(
    "--cloud-name", required=False, help="Name of the cloud to use", default=None
)
@click.option(
    "--idle-timeout",
    required=False,
    help="Idle timeout (in minutes), after which the cluster is stopped. Idle "
    "time is defined as the time during which a cluster is not running a user "
    "command (through 'anyscale exec' or the Web UI), and does not have an "
    "attached driver. Time spent running Jupyter commands, or commands run "
    "through ssh, is still considered 'idle'. -1 means no timeout. "
    "Default: 120 minutes",
    type=int,
)
@click.option("--verbose", "-v", help="Print out more information", is_flag=True)
@click.option(
    "--yes",
    "-y",
    is_flag=True,
    default=False,
    help="DEPRECATED: Cluster will start without confirmation.",
)
@click.option(
    "--no-rapid-start",
    is_flag=True,
    default=False,
    help=(
        "Force bypassing RapidStart for this cluster. "
        "A brand new instance will be acquired from the cloud provider."
    ),
)
def anyscale_up(
    session_name: Optional[str],
    config: Optional[str],
    build_id: Optional[str],
    compute_config: Optional[str],
    min_workers: Optional[int],
    max_workers: Optional[int],
    no_restart: bool,
    restart_only: bool,
    disable_sync: bool,
    cloud_id: Optional[str],
    cloud_name: Optional[str],
    idle_timeout: Optional[int],
    verbose: bool,
    yes: bool,
    no_rapid_start: bool,
    dangerously_set_build_id: Optional[str],
) -> None:
    session_controller = SessionController()
    session_controller.up(
        session_name=session_name,
        config=config,
        build_id=build_id,
        compute_config=compute_config,
        min_workers=min_workers,
        max_workers=max_workers,
        no_restart=no_restart,
        restart_only=restart_only,
        disable_sync=disable_sync,
        cloud_id=cloud_id,
        cloud_name=cloud_name,
        idle_timeout=idle_timeout,
        verbose=verbose,
        no_rapid_start=no_rapid_start,
        dangerously_set_build_id=dangerously_set_build_id,
    )


@click.command(
    name="start",
    context_settings=dict(ignore_unknown_options=True,),
    help="[DEPRECATED] Start or update a cluster based on the current project configuration.",
    hidden=True,
)
@click.argument("session-name", required=False)
@click.option(
    "--build-id",
    help=(
        "The build ID generated from the app templates service. Either the --build-id "
        "or the --build-identifier must be specified."
    ),
    # TODO: add the app templates CLI command to the description
    # once the CLI is done.
    default=None,
    required=False,
)
@click.option(
    "--build-identifier",
    help=(
        "The build identifier for a build generated from the app config service. "
        "A build identifier consists of an app config name and an optional revision "
        "of the build. Eg: 'my_app_config:2' where 2 is the revision number. If no "
        "revision is specified, use the latest revision. Either the --build-id or the "
        "--build-identifier must be specified."
    ),
    default=None,
    required=False,
)
@click.option(
    "--compute-config",
    help=(
        "The JSON file of the compute config to launch this cluster with. "
        "An example can be found at {filename}. "
        "The full JSON schema can be viewed at {website}. ".format(
            filename=COMPUTE_CONFIG_FILENAME, website="api.anyscale.com/v0/docs",
        )
    ),
    required=True,
)
@click.option(
    "--idle-timeout",
    required=False,
    help="Idle timeout (in minutes), after which the cluster is stopped. Idle "
    "time is defined as the time during which a cluster is not running a user "
    "command (through 'anyscale exec' or the Web UI), and does not have an "
    "attached driver. Time spent running Jupyter commands, or commands run "
    "through ssh, is still considered 'idle'. -1 means no timeout. "
    "Default: 120 minutes",
    type=int,
)
@click.option(
    "--sync-files",
    is_flag=True,
    default=False,
    help=("Syncs working directory after the session is started"),
    hidden=True,
)
def anyscale_start(
    session_name: Optional[str],
    build_id: str,
    build_identifier: str,
    compute_config: str,
    idle_timeout: Optional[int],
    sync_files: bool,
) -> None:
    log.warning(
        "`anyscale start` has been deprecated. Please use `anyscale cluster start` instead."
    )
    session_controller = SessionController()
    session_controller.start(
        session_name=session_name,
        compute_config=compute_config,
        build_id=build_id,
        build_identifier=build_identifier,
        idle_timeout=idle_timeout,
        sync_files=sync_files,
    )


@click.command(name="ssh", help="SSH into head node of cluster.", hidden=True)
@click.argument(
    "cluster-name", type=str, required=False, default=None, envvar="CLUSTER_NAME"
)
@click.option("-o", "--ssh-option", multiple=True)
@click.option(
    "-w",
    "--worker-node-id",
    type=str,
    default=None,
    help=(
        "SSH into the given worker node id instead of the head node. "
        "You can also run this command directly on the head node. "
        "ssh_options are ignored when SSHing from the head node "
        "to the worker node."
    ),
)
@click.option(
    "-n",
    "--worker-node-ip",
    type=str,
    default=None,
    help=(
        "SSH into the given worker node ip address instead of the head node. "
        "You can also run this command directly on the head node. "
        "ssh_options are ignored when SSHing from the head node "
        "to the worker node."
    ),
)
@click.option(
    "-h",
    "--force-head-node",
    type=bool,
    default=False,
    hidden=True,
    help=(
        "Force the ssh to the worker node id to assume it's on the "
        "head node when we know it's supposed to be there."
    ),
)
def anyscale_ssh(
    cluster_name: str,
    ssh_option: Tuple[str],
    worker_node_id: Optional[str],
    worker_node_ip: Optional[str],
    force_head_node: bool,
) -> None:
    if not cluster_name:
        cluster_name = os.getenv("SESSION_NAME", "")

    session_controller = SessionController()
    session_controller.ssh(
        cluster_name, ssh_option, worker_node_id, worker_node_ip, force_head_node
    )


@click.command(
    name="autopush",
    short_help="DEPRECATED: Automatically synchronize a local project with a cluster.",
    help="""
DEPRECATED:

This command launches the autopush service that will synchronize
the state of your local project to the Anyscale cluster that you specify.

If there is only a single cluster running, this command without arguments will
default to that cluster.""",
    hidden=True,
)
@click.argument("session-name", type=str, required=False, default=None)
@click.option("--verbose", help="Show output from autopush.", is_flag=True)
@click.option(
    "-d",
    "--delete",
    help="Whether to delete the files that do not exist locally but are found in the cluster.",
    is_flag=True,
    default=False,
)
def anyscale_autopush(
    session_name: Optional[str], verbose: bool, delete: bool,
) -> None:
    session_controller = SessionController()
    session_controller.autopush(session_name, verbose, delete=delete)


@click.command(
    name="autosync",
    short_help="Automatically synchronize a local project with a cluster.",
    help="""
DEPRECATED: Renamed to autopush

This command launches the autosync service that will synchronize
the state of your local project with the Anyscale cluster that you specify.

If there is only a single cluster running, this command without arguments will
default to that cluster.""",
    hidden=True,  # HIDDEN cuz deprecated
)
@click.argument("session-name", type=str, required=False, default=None)
@click.option("--verbose", help="Show output from autosync.", is_flag=True)
@click.option(
    "-d",
    "--delete",
    help="Whether to delete the files that do not exist locally but are found in the cluster.",
    is_flag=True,
    default=False,
)
@click.option(
    "-i",
    "--ignore-errors",
    help="Whether to continue syncing even when rsync returns an error code.",
    is_flag=True,
    default=False,
)
def anyscale_autosync(
    session_name: Optional[str], verbose: bool, delete: bool, ignore_errors: bool,
) -> None:
    raise click.ClickException("Error: autosync has been renamed to autopush")


@click.command(
    name="push", help="DEPRECATED: Push current project to cluster.", hidden=True
)
@click.argument(
    "session-name", type=str, required=False, default=None, envvar="SESSION_NAME"
)
@click.option(
    "--source",
    "-s",
    type=str,
    required=False,
    default=None,
    help="Source location of the files on the local file system which should"
    "be transferred. If source and target are specified, only those files/directories "
    "will be updated.",
)
@click.option(
    "--target",
    "-t",
    type=str,
    required=False,
    default=None,
    help="Target location on the head node to transfer the files to. If source "
    "and target are specified, only those files/directories will be updated.",
)
@click.option(
    "--config",
    type=str,
    required=False,
    default=None,
    help="Updates session with this configuration file.",
)
@click.option(
    "--all-nodes",
    "-A",
    is_flag=True,
    required=False,
    help="[DEPRECATED] Choose to update to all nodes (workers and head) if source and target are specified.",
)
@click.pass_context
def anyscale_push(
    ctx: Any,
    session_name: str,
    source: Optional[str],
    target: Optional[str],
    config: Optional[str],
    all_nodes: bool,
) -> None:
    session_controller = SessionController()
    session_controller.push(session_name, source, target, config, all_nodes)


@click.command(name="pull", help="DEPRECATED: Pull cluster", hidden=True)
@click.argument(
    "session-name", type=str, required=False, default=None, envvar="SESSION_NAME"
)
@click.option(
    "--source",
    "-s",
    type=str,
    required=False,
    default=None,
    help="Source location to transfer files located on head node of cluster "
    "from. If source and target are specified, only those files/directories "
    "will be updated.",
)
@click.option(
    "--target",
    "-t",
    type=str,
    required=False,
    default=None,
    help="Local target location to transfer files to. If source and target "
    "are specified, only those files/directories will be updated.",
)
@click.option(
    "--config",
    type=str,
    required=False,
    default=None,
    help="Pulls cluster configuration from cluster this location.",
)
@click.confirmation_option(
    prompt="Pulling a cluster will override the local project directory. Do you want to continue?"
)
def anyscale_pull(
    session_name: str,
    source: Optional[str],
    target: Optional[str],
    config: Optional[str],
) -> None:
    session_controller = SessionController()
    session_controller.pull(session_name, source, target, config)


@click.command(
    name="fork",
    help="Clones an existing cluster by name and initializes it",
    hidden=True,
)
@click.argument("cluster-name", type=str, required=True)
@click.argument("new-cluster-name", type=str, required=True)
@click.option(
    "--project-name",
    type=str,
    required=False,
    default=None,
    help="""
    Name of the project the existing cluster belongs to.
    By default, we will use the project configured in your current workspace.
    """,
)
def anyscale_fork(
    cluster_name: str, new_cluster_name: str, project_name: Optional[str] = None
) -> None:
    session_controller = SessionController()
    output = session_controller.fork_session(cluster_name, new_cluster_name)
    print(output)



from typing import Optional

import click

from anyscale.controllers.cluster_controller import ClusterController
from anyscale.util import validate_non_negative_arg


@click.group("cluster", help="Interact with clusters on Anyscale.")
def cluster_cli() -> None:
    pass


@cluster_cli.command(
    name="start", help="Start or update and restart a cluster on Anyscale."
)
@click.option(
    "--name",
    "-n",
    required=False,
    default=None,
    help="Name of new or existing cluster to start or update.",
)
@click.option(
    "--env",
    required=False,
    default=None,
    help=(
        "Set the Anyscale app config to use for the cluster. This is a cluster "
        "environment name optionally followed by a colon and a build version number. "
        "Eg: my_cluster_env:1"
    ),
)
@click.option(
    "--compute",
    required=False,
    default=None,
    help=("Name of cluster compute that is already registered with Anyscale."),
)
@click.option(
    "--compute-file",
    required=False,
    default=None,
    help=(
        "The YAML file of the cluster compute config to launch this cluster with. "
        "An example can be found at {website}. ".format(
            website="https://docs.anyscale.com/user-guide/configure/compute-configs",
        )
    ),
)
@click.option(
    "--cluster-id",
    "--id",
    required=False,
    default=None,
    help=(
        "Id of existing cluster to restart. This argument can be used "
        "to interact with any cluster you have access to across projects."
    ),
)
@click.option(
    "--project-id",
    required=False,
    default=None,
    help=(
        "Override id used for this cluster. If not provided, the Anyscale project "
        "context will be used if it exists. Otherwise a default project will be used."
    ),
)
@click.option(
    "--cloud-name",
    required=False,
    default=None,
    help=(
        "Name of cloud to create a default cluster compute with. If a default "
        "cloud needs to be used and this is not provided, the organization default "
        "cloud will be used."
    ),
)
@click.option(
    "--idle-timeout",
    required=False,
    help="Idle timeout (in minutes), after which the cluster is stopped. Idle "
    "time is defined as the time during which a cluster is not running a user "
    "command and does not have an attached driver. Time spent running Jupyter "
    "commands, or commands run through ssh, is still considered "
    "'idle'. -1 means no timeout. Default: 120 minutes",
    type=int,
)
def start(
    name: Optional[str],
    env: Optional[str],
    compute: Optional[str],
    compute_file: Optional[str],
    cluster_id: Optional[str],
    project_id: Optional[str],
    cloud_name: Optional[str],
    idle_timeout: Optional[int],
) -> None:
    cluster_controller = ClusterController()
    cluster_controller.start(
        cluster_name=name,
        cluster_id=cluster_id,
        cluster_env_name=env,
        cluster_compute_name=compute,
        cluster_compute_file=compute_file,
        cloud_name=cloud_name,
        idle_timeout=idle_timeout,
        project_id=project_id,
    )


@cluster_cli.command(name="terminate", help="Terminate a cluster on Anyscale.")
@click.option(
    "--name",
    "-n",
    required=False,
    default=None,
    help="Name of existing cluster to terminate.",
)
@click.option(
    "--cluster-id",
    "--id",
    required=False,
    default=None,
    help=(
        "Id of existing cluster to termiante. This argument can be used "
        "to interact with any cluster you have access to across projects."
    ),
)
@click.option(
    "--project-id",
    required=False,
    default=None,
    help=(
        "Override id used for this cluster. If not provided, the Anyscale project "
        "context will be used if it exists. Otherwise a default project will be used."
    ),
)
def terminate(
    name: Optional[str], cluster_id: Optional[str], project_id: Optional[str],
) -> None:
    cluster_controller = ClusterController()
    cluster_controller.terminate(
        cluster_name=name, cluster_id=cluster_id, project_id=project_id,
    )


@cluster_cli.command(
    name="list",
    help=(
        "List information about clusters on Anyscale. By default only list "
        "active clusters in current project."
    ),
)
@click.option(
    "--name",
    "-n",
    required=False,
    default=None,
    help="Name of existing cluster to get information about.",
)
@click.option(
    "--cluster-id",
    "--id",
    required=False,
    default=None,
    help=(
        "Id of existing cluster get information about. This argument can be used "
        "to interact with any cluster you have access to across projects."
    ),
)
@click.option(
    "--project-id",
    required=False,
    default=None,
    help=(
        "Override id used for this cluster. If not provided, the Anyscale project "
        "context will be used if it exists. Otherwise a default project will be used."
    ),
)
@click.option(
    "--include-all-projects",
    is_flag=True,
    default=False,
    help="List all active clusters user has access to across projects.",
)
@click.option(
    "--include-inactive",
    is_flag=True,
    default=False,
    help="List clusters of all states.",
)
@click.option(
    "--max-items",
    required=False,
    default=20,
    type=int,
    help="Max items to show in list.",
    callback=validate_non_negative_arg,
)
def list(
    name: Optional[str],
    cluster_id: Optional[str],
    project_id: Optional[str],
    include_all_projects: bool,
    include_inactive: bool,
    max_items: int,
) -> None:
    cluster_controller = ClusterController()
    cluster_controller.list(
        cluster_name=name,
        cluster_id=cluster_id,
        project_id=project_id,
        include_all_projects=include_all_projects,
        include_inactive=include_inactive,
        max_items=max_items,
    )
